@@ -54,6 +54,7 @@
 #include "gvfsjobcreatemonitor.h"
 #include "gvfsjobmakedirectory.h"
 #include "gvfsmonitor.h"
+#include "gvfsgphoto2utils.h"
 
 
 /* ------------------------------------------------------------------------------------------------- */
@@ -370,18 +371,6 @@ remove_cache_entry_by_id (GVfsBackendMtp *backend,
 }
 
 
-#define FAIL_DURING_UNMOUNT() \
-  if (g_atomic_int_get (&G_VFS_BACKEND_MTP(backend)->unmount_started)) { \
-    DEBUG ("(I) aborting due to unmount\n"); \
-    g_vfs_job_failed_literal (G_VFS_JOB (job), \
-                              G_IO_ERROR, G_IO_ERROR_CLOSED, \
-                              _("The connection is closed")); \
-    goto exit; \
-  }
-
-
-
-
 /************************************************
  * Initialization
  ************************************************/
@@ -426,11 +415,7 @@ g_vfs_backend_mtp_finalize (GObject *object)
 
   g_hash_table_foreach (backend->monitors, remove_monitor_weak_ref, backend->monitors);
   g_hash_table_unref (backend->monitors);
-
-  /* Leak the mutex if the backend is force unmounted to avoid crash caused by
-   * abort(), when trying to clear already locked mutex. */
-  if (!backend->force_unmounted)
-    g_mutex_clear (&backend->mutex);
+  g_mutex_clear (&backend->mutex);
 
   (*G_OBJECT_CLASS (g_vfs_backend_mtp_parent_class)->finalize) (object);
 
@@ -606,7 +591,6 @@ on_uevent (GUdevClient *client, gchar *action, GUdevDevice *device, gpointer use
                            (char *)path);
     }
 
-    op_backend->force_unmounted = TRUE;
     g_vfs_backend_force_unmount ((GVfsBackend*)op_backend);
   }
 
@@ -790,9 +774,10 @@ mtp_heartbeat (GVfsBackendMtp *backend)
 }
 
 static char *
-get_dev_path_from_host (GVfsJob *job,
-                        GUdevClient *gudev_client,
-                        const char *host)
+get_dev_path_and_device_from_host (GVfsJob *job,
+                                   GUdevClient *gudev_client,
+                                   const char *host,
+                                   GUdevDevice **device)
 {
   /* turn usb:001,041 string into an udev device name */
   if (!g_str_has_prefix (host, "[usb:")) {
@@ -816,15 +801,14 @@ get_dev_path_from_host (GVfsJob *job,
   DEBUG ("(II) get_dev_path_from_host: Parsed '%s' into device name %s\n", host, dev_path);
 
   /* find corresponding GUdevDevice */
-  GUdevDevice *device = g_udev_client_query_by_device_file (gudev_client, dev_path);
-  if (!device) {
+  *device = g_udev_client_query_by_device_file (gudev_client, dev_path);
+  if (!*device) {
     g_free (dev_path);
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
                               _("Couldn't find matching udev device."));
     return NULL;
   }
-  g_object_unref (device);
 
   return dev_path;
 }
@@ -837,6 +821,7 @@ do_mount (GVfsBackend *backend,
            gboolean is_automount)
 {
   GVfsBackendMtp *op_backend = G_VFS_BACKEND_MTP (backend);
+  GUdevDevice *device;
 
   DEBUG ("(I) do_mount\n");
 
@@ -856,7 +841,10 @@ do_mount (GVfsBackend *backend,
     return;
   }
 
-  char *dev_path = get_dev_path_from_host (G_VFS_JOB (job), op_backend->gudev_client, host);
+  char *dev_path = get_dev_path_and_device_from_host (G_VFS_JOB (job),
+                                                      op_backend->gudev_client,
+                                                      host,
+                                                      &device);
   if (dev_path == NULL) {
     g_object_unref (op_backend->gudev_client);
     /* get_dev_path_from_host() sets job state. */
@@ -864,6 +852,11 @@ do_mount (GVfsBackend *backend,
   }
   op_backend->dev_path = dev_path;
 
+  op_backend->volume_name = g_vfs_get_volume_name (device, "ID_MTP");
+  op_backend->volume_icon = g_vfs_get_volume_icon (device);
+  op_backend->volume_symbolic_icon = g_vfs_get_volume_symbolic_icon (device);
+  g_object_unref (device);
+
   op_backend->on_uevent_id =
     g_signal_connect_object (op_backend->gudev_client, "uevent",
                              G_CALLBACK (on_uevent), op_backend, 0);
@@ -933,6 +926,9 @@ do_unmount (GVfsBackend *backend, GVfsJobUnmount *job,
                                op_backend->on_uevent_id);
   g_object_unref (op_backend->gudev_client);
   g_clear_pointer (&op_backend->dev_path, g_free);
+  g_clear_pointer (&op_backend->volume_name, g_free);
+  g_clear_pointer (&op_backend->volume_icon, g_free);
+  g_clear_pointer (&op_backend->volume_symbolic_icon, g_free);
   LIBMTP_Release_Device (op_backend->device);
 
   g_mutex_unlock (&op_backend->mutex);
@@ -1068,16 +1064,20 @@ get_device_info (GVfsBackendMtp *backend, GFileInfo *info)
 
   char *friendlyname = LIBMTP_Get_Friendlyname (device);
   g_file_info_set_display_name (info, friendlyname == NULL ?
-                                      _("Unnamed Device") : friendlyname);
+                                      backend->volume_name : friendlyname);
   free (friendlyname);
 
   g_file_info_set_content_type (info, "inode/directory");
   g_file_info_set_size (info, 0);
 
-  GIcon *icon = g_themed_icon_new ("multimedia-player");
+  GIcon *icon = g_themed_icon_new (backend->volume_icon);
   g_file_info_set_icon (info, icon);
   g_object_unref (icon);
 
+  icon = g_themed_icon_new (backend->volume_symbolic_icon);
+  g_file_info_set_symbolic_icon (info, icon);
+  g_object_unref (icon);
+
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ, TRUE);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
@@ -1117,7 +1117,7 @@ get_device_info (GVfsBackendMtp *backend, GFileInfo *info)
 static void
 get_storage_info (LIBMTP_devicestorage_t *storage, GFileInfo *info) {
 
-  DEBUG_ENUMERATE ("(II) get_storage_info: %X\n", storage->id);
+  DEBUG_ENUMERATE ("(II) get_storage_info: %s\n", storage->id);
 
   char *storage_name = create_storage_name(storage);
   g_file_info_set_name (info, storage_name);
@@ -1261,8 +1261,6 @@ do_enumerate (GVfsBackend *backend,
 
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   LIBMTP_mtpdevice_t *device;
   device = op_backend->device;
 
@@ -1359,8 +1357,6 @@ do_query_info (GVfsBackend *backend,
   LIBMTP_mtpdevice_t *device;
   device = G_VFS_BACKEND_MTP (backend)->device;
 
-  FAIL_DURING_UNMOUNT();
-
   if (ne == 2 && elements[1][0] == '\0') {
     get_device_info (G_VFS_BACKEND_MTP (backend), info);
   } else if (ne < 3) {
@@ -1440,8 +1436,6 @@ do_query_fs_info (GVfsBackend *backend,
   gchar **elements = g_strsplit_set (filename, "/", -1);
   unsigned int ne = g_strv_length (elements);
 
-  FAIL_DURING_UNMOUNT();
-
   LIBMTP_mtpdevice_t *device;
   device = G_VFS_BACKEND_MTP (backend)->device;
 
@@ -1456,7 +1450,6 @@ do_query_fs_info (GVfsBackend *backend,
       g_vfs_job_failed_literal (G_VFS_JOB (job),
                                 G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
                                 _("File not found"));
-      goto exit;
     }
 
     LIBMTP_devicestorage_t *storage;
@@ -1469,7 +1462,6 @@ do_query_fs_info (GVfsBackend *backend,
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
 
- exit:
   g_strfreev (elements);
   g_mutex_unlock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
@@ -1514,8 +1506,6 @@ do_make_directory (GVfsBackend *backend,
   gchar **elements = g_strsplit_set (filename, "/", -1);
   unsigned int ne = g_strv_length (elements);
 
-  FAIL_DURING_UNMOUNT();
-
   if (ne < 3) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
@@ -1579,8 +1569,6 @@ do_pull (GVfsBackend *backend,
   GFileInfo *local_info = NULL;
   GFileInfo *info = NULL;
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), source);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -1729,8 +1717,6 @@ do_push (GVfsBackend *backend,
   gchar **elements = g_strsplit_set (destination, "/", -1);
   unsigned int ne = g_strv_length (elements);
 
-  FAIL_DURING_UNMOUNT();
-
   if (ne < 3) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
@@ -1884,8 +1870,6 @@ do_delete (GVfsBackend *backend,
   DEBUG ("(I) do_delete (filename = %s)\n", filename);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), filename);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -1931,8 +1915,6 @@ do_set_display_name (GVfsBackend *backend,
   DEBUG ("(I) do_set_display_name '%s' --> '%s'\n", filename, display_name);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), filename);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2000,8 +1982,6 @@ do_open_for_read (GVfsBackend *backend,
   DEBUG ("(I) do_open_for_read (%s)\n", filename);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), filename);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2061,8 +2041,6 @@ do_open_icon_for_read (GVfsBackend *backend,
   DEBUG ("(I) do_open_icon_for_read (%s)\n", icon_id);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   guint id = strtol (icon_id, NULL, 16);
 
   if (id > 0) {
@@ -2148,8 +2126,6 @@ do_seek_on_read (GVfsBackend *backend,
   DEBUG ("(I) do_seek_on_read (%X %lu %ld %u)\n", id, old_offset, offset, type);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   if (type == G_SEEK_END) {
     offset = size + offset;
   } else if (type == G_SEEK_CUR) {
@@ -2187,8 +2163,6 @@ do_read (GVfsBackend *backend,
   DEBUG ("(I) do_read (%X %lu %lu)\n", id, offset, bytes_requested);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   uint32_t actual;
   if (handle->handle_type == HANDLE_FILE) {
 #if HAVE_LIBMTP_1_1_6
@@ -2272,8 +2246,6 @@ do_create (GVfsBackend *backend,
   gchar **elements = g_strsplit_set (filename, "/", -1);
   unsigned int ne = g_strv_length (elements);
 
-  FAIL_DURING_UNMOUNT();
-
   if (ne < 3) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_PERMISSION_DENIED,
@@ -2356,8 +2328,6 @@ do_append_to (GVfsBackend *backend,
   DEBUG ("(I) do_append_to (%s)\n", filename);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), filename);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2426,8 +2396,6 @@ do_replace (GVfsBackend *backend,
   DEBUG ("(I) do_replace (%s)\n", filename);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), filename);
   if (entry == NULL) {
     g_mutex_unlock (&G_VFS_BACKEND_MTP (backend)->mutex);
@@ -2497,8 +2465,6 @@ do_write (GVfsBackend *backend,
   DEBUG ("(I) do_write (%X %lu %lu)\n", id, offset, buffer_size);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   int ret = LIBMTP_SendPartialObject (G_VFS_BACKEND_MTP (backend)->device, id, offset,
                                       (unsigned char *)buffer, buffer_size);
   if (ret != 0) {
@@ -2532,8 +2498,6 @@ do_seek_on_write (GVfsBackend *backend,
   DEBUG ("(I) do_seek_on_write (%X %lu %ld %u)\n", id, old_offset, offset, type);
   g_mutex_lock (&G_VFS_BACKEND_MTP (backend)->mutex);
 
-  FAIL_DURING_UNMOUNT();
-
   if (type == G_SEEK_END) {
     offset = size + offset;
   } else if (type == G_SEEK_CUR) {
@@ -2592,8 +2556,6 @@ do_close_write (GVfsBackend *backend,
 
   RWHandle *handle = opaque_handle;
 
-  FAIL_DURING_UNMOUNT();
-
   int ret = LIBMTP_EndEditObject (G_VFS_BACKEND_MTP (backend)->device, handle->id);
   if (ret != 0) {
     fail_job (G_VFS_JOB (job), G_VFS_BACKEND_MTP (backend)->device);
